<template>
  <div
    v-show="data.isShow"
    :class="['d-frame', {'d-frame-full': fullScreen}]"
    ref="dFrame"
    :data-id="data.id"
    @contextmenu.prevent="openMenuList($event.offsetY,$event.offsetX,data)"
    v-bind:style="{left: data.leftOffset + 'px', top: data.topOffset + 'px', 'z-index': 125 + data.order, width: width + 'px', height: height + 'px'}"
  >
    <div class="d-frame-title" v-drag='fullScreen' @click="frameClick">
      <div class="d-frame-title-operation">
        <i class="icon-minus d-frame-operation-minus" @click="minFrame(data,$event)">
          <img :src="`${publicPath}images/desktop/header-icon/min.png`" alt />
        </i>
        <i class="icon-screen-toggle d-frame-operation-full" @click="changeFullScreen($event)">
          <img
            :src="fullScreen?`${publicPath}images/desktop/header-icon/mini.png`:`${publicPath}images/desktop/header-icon/max.png`"
            alt
          />
        </i>
        <i class="icon-close d-frame-operation-close" @click="closeFrame($event)"></i>
      </div>
      <slot name="d-frame-title-content">
        <div class="d-frame-title-content">
          <img :src="data.icon" />
          <span>{{data.title}}</span>
        </div>
      </slot>
    </div>
    <div class="d-frame-content">
      <div class="iframe-mask" v-if="data.order!=1" @click="frameClick"></div>
      <iframe :src="data.url" v-if="data.url" :name="data.name"></iframe>
      <div v-html="data.html" v-if="data.html"></div>
      <!-- <span class="d-frame-operation-resize" v-resize></span> -->
      <span
        class="d-frame-operation-resize"
        @mousedown="mousedown"
        @mousemove.prevent="mousemove"
        @mouseup="mouseup"
      ></span>
    </div>
    <div class="menuList" v-show="showMenu" :style="{top:menuTop+'px',left:menuLeft+'px'}">
      <ul>
        <li @click="refreshApp">重新加载</li>
        <li @click="changeFullScreen">最大化</li>
        <li @click="minFrame">最小化</li>
        <li @click="closeFrame">关闭</li>
      </ul>
    </div>
  </div>
</template>

<script>
import html2canvas from 'html2canvas';
export default {
  name: "DFrame",
  props: {
    data: Object
  },
  data() {
    return {
      publicPath: process.env.BASE_URL,
      showMenu: false,
      menuTop: 0,
      menuLeft: 0,
      fullScreen: false,
      width: this.data.width || 1024,
      height: this.data.height || 512,
      resizeObj: {
        startW: 0,
        startH: 0,
        mouX: 0,
        mouY: 0,
        resizeLock: false
      }
    }
  },
  watch: {
    showMenu(val) {
      if (val) {
        document.body.addEventListener('click', this.closeMenuList)
      } else {
        document.body.removeEventListener('click', this.closeMenuList)
      }
    }
  },
  mounted() {
    console.log(document.querySelectorAll('iframe'))
    window.addEventListener('message', (d) => {
      let { source, trigger, menuT, menuL } = d.data;
      if (trigger == 'contextmenu') {
        this.openMenuList(menuT, menuL)
      }
    });
  },
  methods: {
    openMenuList(t, l, frame) {
      this.showMenu = true;
      this.menuTop = t;
      this.menuLeft = l;
    },
    closeMenuList() {
      this.showMenu = false;
    },
    //resize
    mousedown(e) {
      this.resizeObj.mouX = e.clientX;
      this.resizeObj.mouY = e.clientY;
      this.resizeObj.startW = this.width;
      this.resizeObj.startH = this.height;
      this.resizeObj.resizeLock = true;
    },
    mousemove(e) {
      document.onmousemove = function (e) {
        if (!this.resizeObj.resizeLock) {
          return
        }
        e = e || window.event;
        e.preventDefault();
        let curWidth = this.resizeObj.startW + e.clientX - this.resizeObj.mouX;
        let curHeight = this.resizeObj.startH + e.clientY - this.resizeObj.mouY;

        this.width = curWidth;
        this.height = curHeight;
      }.bind(this)
    },
    mouseup() {
      this.resizeObj.resizeLock = false;
      document.onmousemove = null;
    },
    //resize end
    //重新加载应用
    refreshApp() {
      window.frames[this.data.name].location.reload()
    },
    closeFrame: function () {
      this.$store.dispatch('desktop/closeFrame', this.data);
      //this.refreshDock();
      this.$store.commit('desktop/highlight')

    },
    changeFullScreen: function () {
      this.fullScreen = !this.fullScreen;
    },
    frameClick(e) {
      this.$store.commit('desktop/refreshFrame', this.data);
    },
    minFrame(data, e) {

      //if(data.name=='cameraVideo'||data.name=='search'||data.name=='library'||data.name=='cameraAccess'||data.name=='dataStack'){
      //let shotSrc = canvas.toDataURL();
      this.$store.commit('desktop/addMinDock', {
        id: this.data.id,
        src: this.data.icon,
        alt: this.data.title,
        type: "3",
        //screenshot: shotSrc
      });
      return false;
      //}
      //找到当前的iframe
      //let curIframe = Array.from(document.querySelectorAll('iframe')).find(iframe => iframe.src.indexOf(data.url) >= 0);
      //保存当前应用快照
      // html2canvas(curIframe.contentWindow.document.body, {
      //   dpi: window.devicePixelRatio * 4,
      //   logging: true, //查看html2canvas内部执行流程
      //   removeContainer: true,
      //   imageTimeout: 0,
      //   useCORS: true, //开启跨域配置
      //   //allowTaint: true
      // }).then(canvas => {
      //   let shotSrc = canvas.toDataURL();
      //   this.$store.commit('desktop/addMinDock', {
      //     id: this.data.id,
      //     src: this.data.icon,
      //     alt: this.data.title,
      //     type: "3",
      //     screenshot: shotSrc
      //   });
      // }).catch(e=>{
      //   console.log(e);
      //   this.$store.commit('desktop/addMinDock', {
      //     id: this.data.id,
      //     src: this.data.icon,
      //     alt: this.data.title,
      //     type: "3",
      //     screenshot: ''
      //   });
      // });
      //return false;

    },

  },
  directives: {
    drag(el, binding) {
      if (el.dragLock || binding.arg) {
        return
      }
      el.onmousedown = function (e) {
        console.log('binding',binding)
        if(binding.value) return;
        el.dragLock = true;
        let disx = e.clientX - el.parentElement.offsetLeft;
        let disy = e.clientY - el.parentElement.offsetTop;
        el.style.cursor = 'move';
        document.onmousemove = function (e) {
          e.preventDefault();
          window.getSelection().removeAllRanges();
          let toLeft = e.clientX - disx;
          let toTop = e.clientY - disy;
          //边界处理
          if (toLeft <= -(el.parentElement.offsetWidth - 46 * 3)) {
            toLeft = -(el.parentElement.offsetWidth - 46 * 3)
          } else if (toLeft >= document.body.getBoundingClientRect().width - 46 * 3) {
            toLeft = document.body.getBoundingClientRect().width - 46 * 3;
          }
          el.parentElement.style.left = toLeft + 'px';
          if (toTop <= 40) {
            toTop = 40
          } else if (toTop >= document.body.getBoundingClientRect().height - 31) {
            toTop = document.body.getBoundingClientRect().height - 31;
          }
          el.parentElement.style.top = toTop + 'px';
          
        };
        document.onmouseup = function () {
          el.dragLock = false;
          document.onmousemove = document.onmouseup = null;
          el.style.cursor = 'unset';
        };
      }
    },
    resize(el, binding, vnode) {
      if (el.resizeLock) {
        return;
      }
      el.onmousedown = function (e) {
        el.resizeLock = true;
        let disx = e.clientX;
        let disy = e.clientY;
        let disw = vnode.context.width;
        let dish = vnode.context.height;
        el.style.cursor = 'nw-resize';
        document.onmousemove = function (e) {
          window.getSelection().removeAllRanges();
          vnode.context.width = disw + e.clientX - disx;
          vnode.context.height = dish + e.clientY - disy;
        };
        document.onmouseup = function () {
          el.resizeLock = false;
          document.onmousemove = document.onmouseup = null;
          el.style.cursor = 'unset';
        };
      }
    }
  }
}
</script>

<style scoped>
html,
body {
  height: 100%;
}
.d-frame {
  position: fixed;
  box-shadow: 0 5px 10px rgba(0, 0, 0, 0.5);
}

.d-frame.d-frame-full {
  width: 100% !important;
  height: 100% !important;
  left: 0 !important;
  top: 40px !important;
}

.d-frame .d-frame-title {
  /* min-width: 378px; */
  height: 30px;
  background-color: #e0e0e0;
  border-radius: 4px 4px 0 0;
  border-top: 1px solid #d0d0d0;
  border-left: 1px solid #d0d0d0;
  border-right: 1px solid #d0d0d0;
}
.iframe-mask {
  position: absolute;
  top: 30px;
  left: 0;
  width: 100%;
  height: 100%;
  background: transparent;
}
.d-frame-title .icon-close {
  border-radius: 0 4px 0 0;
  background: url("/images/desktop/header-icon/close.png") no-repeat 50%;
}
.d-frame-title-operation {
  position: absolute;
  top: 0;
  right: 0;
  height: 30px;
  z-index: 2;
}

.d-frame-title-operation i {
  font-size: 18px;
  display: inline-block;
  height: 100%;
  width: 46px;
  text-align: center;
  vertical-align: middle;
  cursor: pointer;
  line-height: 28px;
}

.d-frame-title-operation i:hover {
  background-color: #c7cacf;
}
.d-frame-title-operation .icon-close:hover {
  background-color: red;
  background-image: url("/images/desktop/header-icon/close-hover.png");
}

/* .d-frame-title-operation:hover .d-frame-operation-close::before {
  content: "\f057";
} */

.d-frame-operation-minus {
  color: #fbb450;
}

.d-frame-operation-full {
  color: #00d800;
}

.d-frame-title-content {
  line-height: 30px;
  text-align: center;
  background: #dee1e6;
}

.d-frame-title-content img {
  width: auto;
  height: 18px;
  position: relative;
  top: 2px;
  margin-right: 6px;
}

.d-frame-title-content span {
  position: relative;
  top: -1px;
  font-size: 15px;
  color: #777;
  /* font-family: '黑体'; */
}

.d-frame-content {
  width: 100%;
  height: calc(100% - 30px);
  border-radius: 0 0 4px 4px;
  background-color: #fff;
}
.d-frame-full .d-frame-content{
  height: calc(100% - 71px);
}
.d-frame-content iframe {
  width: 100%;
  height: 100%;
  border: 0;
}

.d-frame-operation-resize {
  position: absolute;
  bottom: 0;
  right: 0;
  width: 15px;
  height: 15px;
  z-index: 1000;
  cursor: nwse-resize;
  background: url("/images/desktop/resize.png") no-repeat;
}
.menuList {
  background: #fff;
  position: absolute;
  z-index: 100;
  width: 180px;
  text-align: left;
  border: 1px solid #ccc;
}
.menuList li {
  line-height: 28px;
  height: 28px;
  padding-left: 24px;
  background: #fff;
}
.menuList li:hover {
  background: rgba(152, 170, 190, 0.7);
}
</style>